@@ -1,14 +1,27 @@
-import warnings
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.animation as animation
-
 from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 from pykin.utils import transform_utils as tf
 
 # Colors of each directions axes. For ex X is green
 directions_colors = ["green", "cyan", "orange"]
 
+def init_3d_figure(name=None, figsize=(15,7.5), dpi= 80):
+    """
+    Initializes 3d figure
+    """
+    fig = plt.figure(name, figsize=figsize, dpi= dpi)
+    ax = fig.add_subplot(111, projection='3d')
+    return fig, ax
+
+
+def show_figure():
+    """
+    Show figure
+    """
+    plt.show()
+
 
 def _check_color_type(color):
     """
@@ -205,7 +218,7 @@ def plot_animation(
         ax.clear()
 
         if visible_obstacles and obstacles:
-            plot_obstacles(obstacles, ax)
+            plot_obstacles(ax, obstacles)
         
         if eef_poses is not None:
             plot_trajectories(ax, eef_poses)
@@ -221,11 +234,10 @@ def plot_animation(
     ani = animation.FuncAnimation(fig, update, np.arange(len(trajectory)), interval=interval, repeat=repeat)
     plt.show()
 
-def plot_obstacles(obstacles, ax):    
+def plot_obstacles(ax, obstacles):    
     """
     Plot obstacles
     """
-    import trimesh
     from pykin.kinematics.transform import Transform
     for key, value in obstacles:
         o_type = value[0]
@@ -236,9 +248,9 @@ def plot_obstacles(obstacles, ax):
             raise TypeError("Check obstacle pose type..")
 
         if o_type == "mesh":
-            plot_mesh(ax, mesh=o_param, A2B=o_pose.h_mat)
+            plot_mesh(ax, mesh=o_param, A2B=o_pose.h_mat, alpha=0.3)
         if o_type == "sphere":
-            plot_sphere(ax, radius=o_param, p=o_pose, alpha=0.8, color='g')
+            plot_sphere(ax, radius=o_param, p=o_pose.pos, alpha=0.8, color='g')
         if o_type == "box":
             A2B = tf.get_h_mat(o_pose)
             plot_box(ax, size=o_param, A2B=A2B, alpha=0.8, color='b')
@@ -394,14 +406,6 @@ def plot_path_planner(path, ax):
 def plot_mesh(ax=None, mesh=None, A2B=np.eye(4),
               s=np.array([1.0, 1.0, 1.0]), ax_s=1, wireframe=False,
               convex_hull=False, alpha=1.0, color="k"):
-
-    try:
-        import trimesh
-    except ImportError:
-        warnings.warn(
-            "Cannot display mesh. Library 'trimesh' not installed.")
-        return ax
-
     vertices = mesh.vertices * s
     vertices = np.hstack((vertices, np.ones((len(vertices), 1))))
     vertices = np.dot(vertices, A2B.T)[:, :3]
@@ -412,17 +416,34 @@ def plot_mesh(ax=None, mesh=None, A2B=np.eye(4),
     surface.set_alpha(alpha)
     ax.add_collection3d(surface)
 
-def init_3d_figure(name=None, figsize=(15,7.5), dpi= 80):
-    """
-    Initializes 3d figure
-    """
-    fig = plt.figure(name, figsize=figsize, dpi= dpi)
-    ax = fig.add_subplot(111, projection='3d')
-    return fig, ax
+
+def plot_normal_vector(ax, vertices, normals, scale=1, linewidths=(1,), edgecolor="red"):
+    
+    if vertices.ndim != 2:
+        vertices = vertices.reshape(1, -1)
+    if normals.ndim != 2:
+        normals = normals.reshape(1, -1)
+
+    ax.quiver(
+        [vertex[0] for vertex in vertices], 
+        [vertex[1] for vertex in vertices], 
+        [vertex[2] for vertex in vertices], 
+        [normal[0]*scale for normal in normals], 
+        [normal[1]*scale for normal in normals], 
+        [normal[2]*scale for normal in normals], linewidths=linewidths, edgecolor=edgecolor)    
 
 
-def show_figure():
-    """
-    Show figure
-    """
-    plt.show()
+def plot_vertices(ax, vertices, s=5, c='k'):
+    if vertices.ndim != 2:
+        vertices = vertices.reshape(1, -1)
+    ax.scatter([x[0] for x in vertices], [x[1] for x in vertices], 
+        [x[2] for x in vertices], s=s, c=c)
+
+
+def plot_line(ax, vertices, linewidth=1):
+    if vertices.ndim != 2:
+        vertices = vertices.reshape(1, -1)
+    ax.plot(
+        [x[0] for x in vertices], 
+        [x[1] for x in vertices],
+        [x[2] for x in vertices], linewidth=linewidth)
